@@ -11,6 +11,7 @@ import tzlocal
 from asn1crypto import cms, pem
 from pyhanko_certvalidator import ValidationContext
 
+import pyhanko.sign.validation.pdf_embedded
 from pyhanko import __version__
 from pyhanko.config import (
     CLIConfig,
@@ -30,15 +31,12 @@ from pyhanko.pdf_utils.layout import LayoutError
 from pyhanko.pdf_utils.reader import PdfFileReader
 from pyhanko.pdf_utils.writer import copy_into_new_writer
 from pyhanko.sign import fields, signers, validation
-from pyhanko.sign.general import (
-    SignatureValidationError,
-    SigningError,
-    load_certs_from_pemder,
-)
+from pyhanko.sign.general import SigningError, load_certs_from_pemder
 from pyhanko.sign.signers import DEFAULT_SIGNER_KEY_USAGE
 from pyhanko.sign.signers.pdf_cms import PdfCMSSignedAttributes
 from pyhanko.sign.timestamps import HTTPTimeStamper
 from pyhanko.sign.validation import RevocationInfoValidationType
+from pyhanko.sign.validation.errors import SignatureValidationError
 from pyhanko.stamp import QRStampStyle, qr_stamp_file, text_stamp_file
 
 __all__ = ['cli']
@@ -347,7 +345,7 @@ def _signature_status(ltv_profile, vc_kwargs, force_revinfo, key_usage_settings,
                       embedded_sig, skip_diff=False):
     if ltv_profile is None:
         vc = ValidationContext(**vc_kwargs)
-        status = validation.validate_pdf_signature(
+        status = pyhanko.sign.validation.validate_pdf_signature(
             embedded_sig, key_usage_settings=key_usage_settings,
             signer_validation_context=vc,
             skip_diff=skip_diff
@@ -387,11 +385,11 @@ def _signature_status_str(status_callback, pretty_print, executive_summary):
     try:
         status = status_callback()
         if executive_summary and not pretty_print:
-            return ('VALID' if status.bottom_line else 'INVALID', status.bottom_line)
+            return 'VALID' if status.bottom_line else 'INVALID', status.bottom_line
         elif pretty_print:
-            return (status.pretty_print_details(), status.bottom_line)
+            return status.pretty_print_details(), status.bottom_line
         else:
-            return (status.summary(), status.bottom_line)
+            return status.summary(), status.bottom_line
     except validation.ValidationInfoReadingError as e:
         msg = (
             'An error occurred while parsing the revocation information '
@@ -399,16 +397,16 @@ def _signature_status_str(status_callback, pretty_print, executive_summary):
         )
         logger.error(msg)
         if pretty_print:
-            return (msg, False)
+            return msg, False
         else:
-            return ('REVINFO_FAILURE', False)
+            return 'REVINFO_FAILURE', False
     except SignatureValidationError as e:
         msg = 'An error occurred while validating this signature: ' + str(e)
         logger.error(msg, exc_info=e)
         if pretty_print:
-            return (msg, False)
+            return msg, False
         else:
-            return ('INVALID', False)
+            return 'INVALID', False
 
 
 # TODO add an option to do LTV, but guess the profile
@@ -595,8 +593,7 @@ def lta_update(ctx, infile, validation_context, trust, trust_replace,
 #  warn if none is present
 
 @signing.command(name='ltvfix',
-                 help='add revocation information for a signature to the '
-                      'document security store')
+                 help='add revocation information for a signature to the DSS')
 @click.argument('infile', type=click.File('r+b'))
 @click.option('--field', help='name of the signature field', required=True)
 @click.option('--timestamp-url', help='URL for timestamp server',
@@ -1130,16 +1127,21 @@ PKCS11_COMMANDS = [
     ('beid', 'use Belgian eID to sign', addsig_beid)
 ]
 
-if pkcs11_available:
-    for args in PKCS11_COMMANDS:
-        _pkcs11_cmd(*args)
-else:
-    def _unavailable():
-        raise click.ClickException(
-            "This subcommand requires python-pkcs11 to be installed."
-        )
-    for name, hlp, fun in PKCS11_COMMANDS:
-        _pkcs11_cmd(name, hlp + ' [dependencies missing]', _unavailable)
+
+def _process_pkcs11_commands():
+    if pkcs11_available:
+        for args in PKCS11_COMMANDS:
+            _pkcs11_cmd(*args)
+    else:
+        def _unavailable():
+            raise click.ClickException(
+                "This subcommand requires python-pkcs11 to be installed."
+            )
+        for name, hlp, fun in PKCS11_COMMANDS:
+            _pkcs11_cmd(name, hlp + ' [dependencies missing]', _unavailable)
+
+
+_process_pkcs11_commands()
 
 
 def _index_page(page):
