@@ -1,14 +1,17 @@
 import binascii
 import uuid
 from enum import Enum
+
 from .generic import (
-    pdf_name, DictionaryObject, NameObject,
-    PdfObject, StreamObject,
+    DictionaryObject,
+    NameObject,
+    PdfObject,
+    StreamObject,
+    pdf_name,
 )
 from .layout import BoxConstraints
 from .reader import PdfFileReader
 
-
 __all__ = [
     'ResourceType', 'ResourceManagementError',
     'PdfResources', 'PdfContent', 'RawContent', 'ImportedPdfPage'
