@@ -32,7 +32,10 @@ from .rw_common import PdfHandler
 logger = logging.getLogger(__name__)
 
 
-__all__ = ['PdfFileReader', 'HistoricalResolver', 'parse_catalog_version']
+__all__ = [
+    'PdfFileReader', 'HistoricalResolver', 'parse_catalog_version',
+    'RawPdfPath', 'process_data_at_eof'
+]
 
 header_regex = re.compile(b'%PDF-(\\d).(\\d)')
 catalog_version_regex = re.compile(r'/(\d).(\d)')
@@ -189,6 +192,22 @@ class XRefCache:
             # this object might simply not have been reclaimed
             pass
 
+    def check_orphaned_freed_objects(self):
+        """
+        Check for orphaned freed objects in the current revision.
+
+        Internal API.
+        """
+        if self._previous_expected_free:
+            orphans = ','.join(
+                f'{k} {v} obj'
+                for k, v in self._previous_expected_free.items()
+            )
+            raise PdfReadError(
+                "Xref table contains orphaned higher generation objects: "
+                + orphans
+            )
+
     def put_ref(self, idnum, generation, start):
         if idnum in self._initially_dead_objects:
             # see comments in free_ref for justification
@@ -432,7 +451,11 @@ class XRefCache:
         self._next_section()
 
 
-def read_object_header(stream, strict):
+class ObjectHeaderReadError(misc.PdfReadError):
+    pass
+
+
+def _read_object_header(stream, strict):
     # Should never be necessary to read out whitespace, since the
     # cross-reference table should put us in the right spot to read the
     # object header.  In reality... some files have stupid cross reference
@@ -440,11 +463,11 @@ def read_object_header(stream, strict):
     extra = False
     misc.skip_over_comment(stream)
     extra |= misc.skip_over_whitespace(stream)
-    stream.seek(-1, os.SEEK_CUR)
-    idnum = misc.read_until_whitespace(stream)
+    idnum_bytes = misc.read_until_whitespace(stream)
+    idnum = int(idnum_bytes)
     extra |= misc.skip_over_whitespace(stream)
-    stream.seek(-1, os.SEEK_CUR)
-    generation = misc.read_until_whitespace(stream)
+    generation_bytes = misc.read_until_whitespace(stream)
+    generation = int(generation_bytes)
     stream.read(3)
     misc.read_non_whitespace(stream, seek_back=True)
 
@@ -453,13 +476,26 @@ def read_object_header(stream, strict):
             f"Superfluous whitespace found in object header "
             f"{idnum} {generation}"
         )
-    return int(idnum), int(generation)
+    return idnum, generation
+
+
+def read_object_header(stream, strict):
+    pos = stream.tell()
+    try:
+        return _read_object_header(stream, strict)
+    except ValueError as e:
+        raise ObjectHeaderReadError(
+            f"Failed to read object header at {pos}"
+        ) from e
 
 
 def process_data_at_eof(stream) -> int:
     """
     Auxiliary function that reads backwards from the current position
     in a stream to find the EOF marker and startxref value
+
+    This is internal API.
+
     :param stream:
         A stream to read from
     :return:
@@ -588,12 +624,44 @@ class TrailerDictionary(generic.PdfObject):
         )
 
 
+def _attempt_startxref_correction(stream, startxref):
+    # couldn't process data at location pointed to by startxref.
+    # Let's see if we can find the xref table nearby, as we've observed this
+    # error with an off-by-one before.
+    stream.seek(-11, os.SEEK_CUR)
+    tmp = stream.read(20)
+    xref_loc = tmp.find(b"xref")
+    if xref_loc != -1:
+        return startxref - (10 - xref_loc)
+    # No explicit xref table, try finding a cross-reference stream, with a
+    # negative offset to account for the startxref being wrong in either
+    # direction
+    xrefstm_readback = 5
+    stream.seek(startxref - xrefstm_readback)
+    tmp = stream.read(2 * xrefstm_readback)
+    newline_loc = tmp.rfind(b"\n")
+    if newline_loc != -1:
+        stream.seek(startxref - xrefstm_readback + newline_loc - 1)
+        misc.skip_over_whitespace(stream)
+        line_start = stream.tell()
+        for look in range(5):
+            if stream.read(1).isdigit():
+                # This is not a standard PDF, consider adding a warning
+                return line_start + look
+
+    # no xref table found at specified location
+    raise PdfReadError(
+        "Could not find xref table at specified location"
+    )
+
+
 class PdfFileReader(PdfHandler):
     """Class implementing functionality to read a PDF file and cache
     certain data about it."""
 
     last_startxref = None
     has_xref_stream = False
+    err_limit = 10
 
     def __init__(self, stream, strict=True):
         """
@@ -646,15 +714,27 @@ class PdfFileReader(PdfHandler):
         # This is an xref to a stream, so its type better be a stream
         assert stream['/Type'] == '/ObjStm'
         # /N is the number of indirect objects in the stream
-        assert idx < stream['/N']
+        if not (0 <= idx < stream['/N']):
+            if self.strict:
+                raise PdfReadError("Object stream does not contain index")
+            else:
+                return generic.NullObject()
         stream_data = BytesIO(stream.data)
         first_object = stream['/First']
         for i in range(stream['/N']):
-            misc.read_non_whitespace(stream_data, seek_back=True)
-            objnum = generic.NumberObject.read_from_stream(stream_data)
-            misc.read_non_whitespace(stream_data, seek_back=True)
-            offset = generic.NumberObject.read_from_stream(stream_data)
-            misc.read_non_whitespace(stream_data, seek_back=True)
+            try:
+                misc.read_non_whitespace(stream_data, seek_back=True)
+                objnum = generic.NumberObject.read_from_stream(stream_data)
+                misc.read_non_whitespace(stream_data, seek_back=True)
+                offset = generic.NumberObject.read_from_stream(stream_data)
+                misc.read_non_whitespace(stream_data, seek_back=True)
+            except ValueError:
+                if self.strict:
+                    raise PdfReadError(
+                        "Object stream header possibly corrupted"
+                    )
+                else:
+                    return generic.NullObject()
             if objnum != idnum:
                 # We're only interested in one object
                 continue
@@ -666,7 +746,7 @@ class PdfFileReader(PdfHandler):
                 obj = generic.read_object(
                     stream_data, generic.Reference(idnum, 0, self),
                 )
-            except misc.PdfStreamError as e:
+            except (misc.PdfStreamError, misc.PdfReadError) as e:
                 # Stream object cannot be read. Normally, a critical error, but
                 # Adobe Reader doesn't complain, so continue (in strict mode?)
                 logger.warning(
@@ -677,14 +757,26 @@ class PdfFileReader(PdfHandler):
                     raise PdfReadError("Can't read object stream: %s" % e)
                 # Replace with null. Hopefully it's nothing important.
                 obj = generic.NullObject()
+            if isinstance(obj, (generic.StreamObject, generic.IndirectObject)):
+                if self.strict:
+                    raise PdfReadError(
+                        "Encountered forbidden object type in object stream"
+                    )
+                else:
+                    return obj
+
             generic.read_non_whitespace(
                 stream_data, seek_back=True, allow_eof=True
             )
             return obj
 
         if self.strict:
-            raise PdfReadError("This is a fatal error in strict mode.")
-        return generic.NullObject()
+            raise PdfReadError(
+                "Object not found in stream, "
+                "this is a fatal error in strict mode"
+            )
+        else:
+            return generic.NullObject()
 
     def _get_encryption_params(self) -> Optional[generic.DictionaryObject]:
         try:
@@ -890,10 +982,24 @@ class PdfFileReader(PdfHandler):
         self.trailer.container_ref = generic.TrailerReference(self)
         startxref = self.last_startxref
         xref_location_log = self.xrefs.xref_locations
+        # Tracks the number of times we retried to read a particular xref
+        # section
+        err_count = 0
         while startxref is not None:
-            xref_location_log.append(startxref)
+            if (self.strict and err_count) or err_count > self.err_limit:
+                raise PdfReadError("Failed to locate xref section")
+            if not err_count:
+                xref_location_log.append(startxref)
             # load the xref table
             stream.seek(startxref)
+            if misc.skip_over_whitespace(stream):
+                # This is common in linearised files, so we're not marking
+                # this as an error, even in strict mode.
+                logger.debug(
+                    "Encountered unexpected whitespace when looking "
+                    "for xref stream"
+                )
+                startxref = stream.tell()
             x = stream.read(1)
             if x == b"x":
                 # standard cross-reference table
@@ -904,43 +1010,25 @@ class PdfFileReader(PdfHandler):
             elif x.isdigit():
                 # PDF 1.5+ Cross-Reference Stream
                 stream.seek(-1, os.SEEK_CUR)
-                startxref = self._read_xref_stream()
+                try:
+                    startxref = self._read_xref_stream()
+                except ObjectHeaderReadError as e:
+                    logger.debug(
+                        "Failed to read xref stream header, attempting to "
+                        "correct...", exc_info=e
+                    )
+                    # can be caused by an OBO error (pointing too far)
+                    startxref = _attempt_startxref_correction(stream, startxref)
+                    err_count += 1
+                    continue
                 self.has_xref_stream = True
             else:
-                # bad xref character at startxref.  Let's see if we can find
-                # the xref table nearby, as we've observed this error with an
-                # off-by-one before.
-                stream.seek(-11, os.SEEK_CUR)
-                tmp = stream.read(20)
-                xref_loc = tmp.find(b"xref")
-                if xref_loc != -1:
-                    startxref -= (10 - xref_loc)
-                    continue
-                # No explicit xref table, try finding a cross-reference stream.
-                stream.seek(startxref)
-                found = False
-                for look in range(5):
-                    if stream.read(1).isdigit():
-                        # This is not a standard PDF, consider adding a warning
-                        startxref += look
-                        found = True
-                        break
-                if found:
-                    continue
-                # no xref table found at specified location
-                raise PdfReadError(
-                    "Could not find xref table at specified location"
-                )
+                startxref = _attempt_startxref_correction(stream, startxref)
+                err_count += 1
+                continue
+            err_count = 0
 
-        if self.xrefs._previous_expected_free:
-            orphans = ','.join(
-                f'{k} {v} obj'
-                for k, v in self.xrefs._previous_expected_free.items()
-            )
-            raise PdfReadError(
-                "Xref table contains orphaned higher generation objects: "
-                + orphans
-            )
+        self.xrefs.check_orphaned_freed_objects()
 
     def read(self):
         # first, read the header & PDF version number
@@ -1071,7 +1159,8 @@ class PdfFileReader(PdfHandler):
         if self._embedded_signatures is not None:
             return self._embedded_signatures
         from pyhanko.sign.fields import enumerate_sig_fields
-        from pyhanko.sign.validation import EmbeddedPdfSignature
+
+        from ..sign.validation import EmbeddedPdfSignature
         sig_fields = enumerate_sig_fields(self, filled_status=True)
 
         result = sorted(
@@ -1121,6 +1210,8 @@ def convert_to_int(d, size):
 class RawPdfPath:
     """
     Class to model raw paths in a file.
+
+    This class is internal API.
     """
 
     def __init__(self, *path: Union[str, int]):
@@ -1468,15 +1559,15 @@ class HistoricalResolver(PdfHandler):
             # separately.
 
             if isinstance(obj, generic.IndirectObject):
-                ref = obj.reference
-                if ref in seen_in_path:
+                obj_ref = obj.reference
+                if obj_ref in seen_in_path:
                     return
-                collected[ref].add(cur_path)
-                seen_in_path = seen_in_path.cons(ref)
-                obj = self(ref)
-                if not is_page_tree and ref in page_tree_objs:
+                collected[obj_ref].add(cur_path)
+                seen_in_path = seen_in_path.cons(obj_ref)
+                obj = self(obj_ref)
+                if not is_page_tree and obj_ref in page_tree_objs:
                     return
-                if not is_struct_tree and ref in struct_tree_objs:
+                if not is_struct_tree and obj_ref in struct_tree_objs:
                     return
             if isinstance(obj, generic.DictionaryObject):
                 for k, v in obj.items():
@@ -1568,7 +1659,8 @@ class HistoricalResolver(PdfHandler):
         struct_tree_nodes = set()
         try:
             struct_tree_root_ref = self.root.raw_get('/StructTreeRoot')
-            for ref in _collect_struct_tree_refs(struct_tree_root_ref.get_object()):
+            struct_tree_root = struct_tree_root_ref.get_object()
+            for ref in _collect_struct_tree_refs(struct_tree_root):
                 if ref in struct_tree_nodes:
                     raise misc.PdfReadError(
                         "Circular reference in structure tree in mapping stage"
