@@ -6,9 +6,10 @@ This module defines a very thin convenience wrapper around
 read the appropriate certificates on the device.
 """
 
-from . import pkcs11 as sign_pkcs11
 from pkcs11 import Session
 
+from . import pkcs11 as sign_pkcs11
+
 __all__ = ['open_beid_session', 'BEIDSigner']
 
 
@@ -44,9 +45,10 @@ class BEIDSigner(sign_pkcs11.PKCS11Signer):
     """
 
     def __init__(self, pkcs11_session: Session, use_auth_cert: bool = False,
-                 bulk_fetch: bool = False):
+                 bulk_fetch: bool = False, embed_roots=True):
         super().__init__(
             pkcs11_session=pkcs11_session,
             cert_label='Authentication' if use_auth_cert else 'Signature',
-            other_certs_to_pull=('Root', 'CA'), bulk_fetch=bulk_fetch
+            other_certs_to_pull=('Root', 'CA'), bulk_fetch=bulk_fetch,
+            embed_roots=embed_roots
         )
