@@ -9,6 +9,7 @@ from dataclasses import dataclass
 from datetime import datetime
 from typing import IO, Iterable, List, Optional, Union
 
+import tzlocal
 from asn1crypto import algos, cms, core, keys
 from asn1crypto import pdf as asn1_pdf
 from asn1crypto import x509
@@ -32,7 +33,6 @@ from pyhanko.sign.general import (
     CertificateStore,
     SigningError,
     SimpleCertificateStore,
-    _process_pss_params,
     _translate_pyca_cryptography_cert_to_asn1,
     _translate_pyca_cryptography_key_to_asn1,
     get_pyca_cryptography_hash,
@@ -40,6 +40,7 @@ from pyhanko.sign.general import (
     load_certs_from_pemder,
     load_private_key_from_pemder,
     optimal_pss_params,
+    process_pss_params,
     simple_cms_attribute,
 )
 from pyhanko.sign.timestamps import TimeStamper
@@ -94,10 +95,19 @@ def _ensure_content_type(encap_content_info):
     return encap_content_info, content_type
 
 
-def _cms_version(content_type: Union[str, core.ObjectIdentifier]):
+def _cms_version(content_type: Union[str, core.ObjectIdentifier],
+                 has_attribute_certs):
+
+    if has_attribute_certs:
+        return 'v4'
+
     if isinstance(content_type, core.ObjectIdentifier):
         content_type = content_type.native
 
+    # We don't support signing with subjectKeyIdentifier, so there's no
+    # need to take that into account here. We also don't do version 1 attribute
+    # certificates.
+    # (these matter for distinguishing v1<>v3)
     return 'v1' if content_type == 'data' else 'v3'
 
 
@@ -259,6 +269,14 @@ class Signer:
     The (cryptographic) signature mechanism to use.
     """
 
+    attribute_certs: Iterable[cms.AttributeCertificateV2] = ()
+    """
+    Attribute certificates to include with the signature.
+
+    .. note::
+        Only ``v2`` attribute certificates are supported.
+    """
+
     def __init__(self, prefer_pss=False, embed_roots=True):
         self.prefer_pss = prefer_pss
         self.embed_roots = embed_roots
@@ -370,24 +388,29 @@ class Signer:
 
     def _signed_attr_providers(self, data_digest: bytes, digest_algorithm: str,
                                attr_settings: PdfCMSSignedAttributes,
-                               timestamper=None, use_pades=False):
+                               timestamper=None, use_cades=False,
+                               is_pdf_sig=True):
         """
         Prepare "standard" signed attribute providers. Internal API.
         """
         yield attributes.SigningCertificateV2Provider(
             signing_cert=self.signing_cert
         )
-        if not use_pades:
+        if not is_pdf_sig or not use_cades:
+            # NOTE: PAdES actually forbids this, but CAdES requires it!
             signing_time = attr_settings.signing_time
+            if signing_time is None and not is_pdf_sig and use_cades:
+                # Ensure CAdES mandate is followed
+                signing_time = datetime.now(tz=tzlocal.get_localzone())
             if signing_time is not None:
-                # NOTE: PAdES actually forbids this!
                 yield attributes.SigningTimeProvider(timestamp=signing_time)
+        if not use_cades:
             if attr_settings.adobe_revinfo_attr is not None:
                 yield attributes.AdobeRevinfoProvider(
                     value=attr_settings.adobe_revinfo_attr
                 )
 
-            # TODO not sure if PAdES allows this, need to check.
+            # TODO not sure if PAdES/CAdES allow this, need to check.
             #  It *should*, but perhaps the version of CMS it is based on is too
             #  old, or it might not allow undefined signed attributes.
             # In the meantime, we only add this attribute to non-PAdES sigs
@@ -469,7 +492,10 @@ class Signer:
 
         # Note: we do not add the TS certs at this point
         if self.embed_roots:
-            certs = set(self.cert_registry)
+            certs = {
+                cms.CertificateChoices(name='certificate', value=cert)
+                for cert in self.cert_registry
+            }
         else:
             # asn1crypto's heuristic is good enough for now, we won't check the
             # actual signatures. CAs that make use of self-issued certificates
@@ -477,10 +503,17 @@ class Signer:
             # distinguish between different certs, which will be picked up by
             # asn1crypto either way.
             certs = {
-                cert for cert in self.cert_registry
-                if cert.self_signed == 'no'
+                cms.CertificateChoices(name='certificate', value=cert)
+                for cert in self.cert_registry if cert.self_signed == 'no'
             }
-        certs.add(self.signing_cert)
+        certs.add(
+            cms.CertificateChoices(name='certificate', value=self.signing_cert)
+        )
+
+        # add attribute certs
+        for ac in self.attribute_certs:
+            certs.add(cms.CertificateChoices(name='v2_attr_cert', value=ac))
+
         # this is the SignedData object for our message (see RFC 2315 § 9.1)
         signed_data = {
             'version': cms_version,
@@ -568,13 +601,14 @@ class Signer:
             digest_algorithm=digest_algorithm, signature=signature,
             timestamper=timestamper
         )
-        return await format_attributes(list(provs), dry_run=dry_run)
+        attrs = await format_attributes(list(provs), dry_run=dry_run)
+        return attrs or None
 
     async def signed_attrs(self, data_digest: bytes,
                            digest_algorithm: str,
                            attr_settings: PdfCMSSignedAttributes = None,
                            content_type='data', use_pades=False,
-                           timestamper=None, dry_run=False):
+                           timestamper=None, dry_run=False, is_pdf_sig=True):
         """
         .. versionchanged:: 0.4.0
             Added positional ``digest_algorithm`` parameter _(breaking change)_.
@@ -613,6 +647,11 @@ class Signer:
             .. danger::
                 This parameter is internal API, and non-default values must not
                 be used to produce PDF signatures.
+        :param is_pdf_sig:
+            Whether the signature being generated is for use in a PDF document.
+
+            .. danger::
+                This parameter is internal API.
         :return:
             An :class:`.asn1crypto.cms.CMSAttributes` object.
         """
@@ -621,8 +660,8 @@ class Signer:
 
         provs = self._signed_attr_providers(
             data_digest=data_digest, digest_algorithm=digest_algorithm,
-            use_pades=use_pades, attr_settings=attr_settings,
-            timestamper=timestamper
+            use_cades=use_pades, attr_settings=attr_settings,
+            timestamper=timestamper, is_pdf_sig=is_pdf_sig
         )
 
         return await format_signed_attributes(
@@ -633,7 +672,8 @@ class Signer:
     async def async_sign(self, data_digest: bytes, digest_algorithm: str,
                          dry_run=False, use_pades=False, timestamper=None,
                          signed_attr_settings: PdfCMSSignedAttributes = None,
-                         encap_content_info=None) -> cms.ContentInfo:
+                         is_pdf_sig=True, encap_content_info=None) \
+            -> cms.ContentInfo:
 
         """
         .. versionadded:: 0.9.0
@@ -669,6 +709,11 @@ class Signer:
                 this might still hit the timestamping server (in order to
                 produce a realistic size estimate), but the dummy response will
                 be cached.
+        :param is_pdf_sig:
+            Whether the signature being generated is for use in a PDF document.
+
+            .. danger::
+                This parameter is internal API.
         :param encap_content_info:
             Data to encapsulate in the CMS object.
 
@@ -684,11 +729,11 @@ class Signer:
         signed_attrs = await self.signed_attrs(
             data_digest, digest_algorithm, attr_settings=signed_attr_settings,
             use_pades=use_pades, timestamper=timestamper,
-            dry_run=dry_run, content_type=content_type
+            dry_run=dry_run, content_type=content_type, is_pdf_sig=is_pdf_sig
         )
         return await self.async_sign_prescribed_attributes(
             digest_algorithm, signed_attrs,
-            cms_version=_cms_version(content_type),
+            cms_version=_cms_version(content_type, bool(self.attribute_certs)),
             dry_run=dry_run, timestamper=timestamper,
             encap_content_info=encap_content_info
         )
@@ -786,10 +831,13 @@ class Signer:
                 and :class:`cms.EncapsulatedContentInfo` otherwise.
 
             .. warning::
-                We currently only support CMS v1 and v3 signatures.
-                This is only a concern if you need attribute certificate
-                support, in which case you can override the CMS version number
+                We currently only support CMS v1, v3 and v4 signatures.
+                This is only a concern if you need certificates or CRLs
+                of type 'other', in which case you can change the version
                 yourself (this will not invalidate any signatures).
+                You'll also need to do this if you need support for version 1
+                attribute certificates, or if you want to sign with
+                ``subjectKeyIdentifier`` in the ``sid`` field.
         :param digest_algorithm:
             The name of the digest algorithm to use.
         :param detached:
@@ -822,7 +870,7 @@ class Signer:
         )
         return await self.async_sign(
             data_digest=digest_bytes, digest_algorithm=digest_algorithm,
-            use_pades=use_cades,
+            use_pades=use_cades, is_pdf_sig=False,
             timestamper=timestamper, encap_content_info=encap_content_info,
             signed_attr_settings=signed_attr_settings
         )
@@ -1006,10 +1054,13 @@ class Signer:
                 and :class:`cms.EncapsulatedContentInfo` otherwise.
 
             .. warning::
-                We currently only support CMS v1 and v3 signatures.
-                This is only a concern if you need attribute certificate
-                support, in which case you can override the CMS version number
+                We currently only support CMS v1, v3 and v4 signatures.
+                This is only a concern if you need certificates or CRLs
+                of type 'other', in which case you can change the version
                 yourself (this will not invalidate any signatures).
+                You'll also need to do this if you need support for version 1
+                attribute certificates, or if you want to sign with
+                ``subjectKeyIdentifier`` in the ``sid`` field.
         :param digest_algorithm:
             The name of the digest algorithm to use.
         :param detached:
@@ -1096,11 +1147,13 @@ class SimpleSigner(Signer):
                  signing_key: keys.PrivateKeyInfo,
                  cert_registry: CertificateStore,
                  signature_mechanism: SignedDigestAlgorithm = None,
-                 prefer_pss=False, embed_roots=True):
+                 prefer_pss=False, embed_roots=True, attribute_certs=None):
         self.signing_cert = signing_cert
         self.signing_key = signing_key
         self.cert_registry = cert_registry
         self.signature_mechanism = signature_mechanism
+        if attribute_certs is not None:
+            self.attribute_certs = list(attribute_certs)
         super().__init__(prefer_pss=prefer_pss, embed_roots=embed_roots)
 
     async def async_sign_raw(self, data: bytes, digest_algorithm: str,
@@ -1132,7 +1185,7 @@ class SimpleSigner(Signer):
             return priv_key.sign(data, padding, hash_algo)
         elif mechanism == 'rsassa_pss':
             params = signature_mechanism['parameters']
-            padding, hash_algo = _process_pss_params(params, digest_algorithm)
+            padding, hash_algo = process_pss_params(params, digest_algorithm)
             assert isinstance(priv_key, RSAPrivateKey)
             return priv_key.sign(data, padding, hash_algo)
         elif mechanism == 'ecdsa':
