@@ -17,6 +17,7 @@ from cryptography.hazmat.primitives import hashes
 from pyhanko_certvalidator import CertificateValidator, ValidationContext
 from pyhanko_certvalidator.errors import PathBuildingError, PathValidationError
 from pyhanko_certvalidator.path import ValidationPath
+from pyhanko_certvalidator.validate import ACValidationResult, async_validate_ac
 
 from pyhanko.pdf_utils import generic, misc
 from pyhanko.pdf_utils.generic import pdf_name
@@ -253,13 +254,13 @@ class PdfSignatureMetadata:
     Specification for a PDF signature.
     """
 
-    field_name: str = None
+    field_name: Optional[str] = None
     """
     The name of the form field to contain the signature.
     If there is only one available signature field, the name may be inferred.
     """
 
-    md_algorithm: str = None
+    md_algorithm: Optional[str] = None
     """
     The name of the digest algorithm to use.
     It should be supported by `pyca/cryptography`.
@@ -268,17 +269,17 @@ class PdfSignatureMetadata:
     a suitable default, unless a seed value dictionary happens to be available.
     """
 
-    location: str = None
+    location: Optional[str] = None
     """
     Location of signing.
     """
 
-    reason: str = None
+    reason: Optional[str] = None
     """
     Reason for signing (textual).
     """
 
-    name: str = None
+    name: Optional[str] = None
     """
     Name of the signer. This value is usually not necessary to set, since
     it should appear on the signer's certificate, but there are cases
@@ -294,7 +295,7 @@ class PdfSignatureMetadata:
     """
     # TODO Does this restriction also apply to prior document timestamps?
 
-    subfilter: SigSeedSubFilter = None
+    subfilter: Optional[SigSeedSubFilter] = None
     """
     Signature subfilter to use.
 
@@ -342,13 +343,13 @@ class PdfSignatureMetadata:
     See also :meth:`.PdfTimeStamper.update_archival_timestamp_chain`.
     """
 
-    timestamp_field_name: str = None
+    timestamp_field_name: Optional[str] = None
     """
     Name of the timestamp field created when :attr:`use_pades_lta` is ``True``.
     If not specified, a unique name will be generated using :mod:`uuid`.
     """
 
-    validation_context: ValidationContext = None
+    validation_context: Optional[ValidationContext] = None
     """
     The validation context to use when validating signatures.
     If provided, the signer's certificate and any timestamp certificates
@@ -410,6 +411,13 @@ class PdfSignatureMetadata:
         predictable.
     """
 
+    ac_validation_context: Optional[ValidationContext] = None
+    """
+    .. versionadded:: 0.11.0
+
+    Validation context for attribute certificates
+    """
+
 
 def _ensure_esic_ext(pdf_writer: BasePdfFileWriter):
     """
@@ -752,13 +760,9 @@ class PdfTimeStamper:
         # in the current chain is valid.
         # TODO: add an option to validate the entire timestamp chain
         #  plus all signatures
-        from pyhanko.sign.validation import (
-            DocumentSecurityStore,
-            _establish_timestamp_trust,
-            get_timestamp_chain,
-        )
+        from .. import validation
 
-        timestamps = get_timestamp_chain(reader)
+        timestamps = validation.get_timestamp_chain(reader)
         try:
             last_timestamp = next(timestamps)
         except StopIteration:
@@ -780,7 +784,7 @@ class PdfTimeStamper:
             expected_imprint = last_timestamp.external_digest
 
             # run validation logic
-            tst_status = await _establish_timestamp_trust(
+            tst_status = await validation.ltv.establish_timestamp_trust(
                 tst_token, validation_context, expected_imprint
             )
 
@@ -797,7 +801,7 @@ class PdfTimeStamper:
         pdf_out = IncrementalPdfFileWriter(res_output)
         if last_timestamp is not None:
             # update the DSS
-            DocumentSecurityStore.supply_dss_in_writer(
+            validation.DocumentSecurityStore.supply_dss_in_writer(
                 pdf_out, last_timestamp.pkcs7_content,
                 paths=(tst_status.validation_path,),
                 validation_context=validation_context,
@@ -1345,6 +1349,11 @@ class PreSignValidationStatus:
     List of CRLS collected so far.
     """
 
+    ac_validation_paths: Optional[List[ValidationPath]] = None
+    """
+    List of validation paths relevant for embedded attribute certificates.
+    """
+
 
 class PdfSigningSession:
     """
@@ -1436,11 +1445,26 @@ class PdfSigningSession:
         timestamper = self.timestamper
         # Finally, fetch validation information for the TSA that we're going to
         # use for our own TS
-        ts_paths = None
         if timestamper is not None:
-            ts_paths = timestamper.validation_paths(validation_context)
-            async for ts_path in ts_paths:
+            async_ts_paths = timestamper.validation_paths(validation_context)
+            ts_paths = []
+            async for ts_path in async_ts_paths:
                 validation_paths.append(ts_path)
+                ts_paths.append(ts_path)
+        else:
+            ts_paths = None
+
+        # fetch attribute certificate validation paths
+        if signature_meta.ac_validation_context is not None:
+            async_aa_paths = self._perform_presign_ac_validation(
+                signature_meta.ac_validation_context
+            )
+            aa_paths = []
+            async for aa_path in async_aa_paths:
+                validation_paths.append(aa_path)
+                aa_paths.append(aa_path)
+        else:
+            aa_paths = None
 
         # do we need adobe-style revocation info?
         if signature_meta.embed_validation_info and not self.use_pades:
@@ -1458,9 +1482,30 @@ class PdfSigningSession:
             ts_validation_paths=ts_paths,
             adobe_revinfo_attr=revinfo,
             ocsps_to_embed=validation_context.ocsps,
-            crls_to_embed=validation_context.crls
+            crls_to_embed=validation_context.crls,
+            ac_validation_paths=aa_paths
         )
 
+    async def _perform_presign_ac_validation(self, validation_context):
+        signer = self.pdf_signer.signer
+        attr_certs = list(signer.attribute_certs)
+        cades_attr_spec = self.pdf_signer.signature_meta.cades_signed_attr_spec
+        # also make sure to pull in the validation chains for all attribute
+        # certificates included in the signer-attributes-v2 attr, if there is
+        # one.
+        if cades_attr_spec is not None and \
+                cades_attr_spec.signer_attributes is not None:
+            attr_certs.extend(cades_attr_spec.signer_attributes.certified_attrs)
+        ac_jobs = [
+            async_validate_ac(
+                ac, validation_context,
+                holder_cert=signer.signing_cert
+            ) for ac in attr_certs
+        ]
+        for ac_job in asyncio.as_completed(ac_jobs):
+            result: ACValidationResult = await ac_job
+            yield result.aa_path
+
     async def _perform_presign_signer_validation(self, validation_context,
                                                  key_usage):
 
